from typing import Any

def conveyor(*args, **kwargs) -> Any:
    """
    () -> array float{facet,dim}.
    Get a numpy array of the facet conveyor velocity.
    """
    pass

def extra(*args, **kwargs) -> Any:
    """
    (slot: int) -> array float{facet} or float{facet}.
    Get the facet extra data in the given slot as an array.
    Extra variables accessed by array must be of type float or vec.
    """
    pass

def fill_conveyor(*args, **kwargs) -> Any:
    """
    (data: array float{facet,dim}) -> None.
    Fill an existing array with the facet conveyor velocity.
    The array must be the correct shape.
    """
    pass

def fill_normal(*args, **kwargs) -> Any:
    """
    (data: array float{facet,dim}) -> None.
    Fill an existing array with the facet normal.
    The array must be the correct shape.
    """
    pass

def fill_pos(*args, **kwargs) -> Any:
    """
    (data: array float{facet,2}) -> None.
    Fill an existing array with the facet location.
    The array must be the correct shape.
    """
    pass

def ids(*args, **kwargs) -> Any:
    """
    () -> array int{facet}.
    Get the facet ids as an array.
    """
    pass

def in_group(*args, **kwargs) -> Any:
    """
    (group_name: str, slot=1) -> array bool{facet}.
    Return facet group membership as a Boolean array.
    """
    pass

def normal(*args, **kwargs) -> Any:
    """
    () -> array float{facet,dim}.
    Get a numpy array of the facet normal.
    """
    pass

def pos(*args, **kwargs) -> Any:
    """
    () -> array float{facet,2}.
    Get a numpy array of the facet location.
    """
    pass

def set_conveyor(*args, **kwargs) -> Any:
    """
    ((data: array float{facet,dim}) -> None.
    Set the facet conveyor velocity from an array.
    """
    pass

def set_extra(*args, **kwargs) -> Any:
    """
    (slot: int, data: array float{facet} or float{facet}) -> None.
    Set the facet extra data in the given slot with an array.
    Extra variables set by array must be of type float or vec.
    """
    pass

def set_group(*args, **kwargs) -> Any:
    """
    (membership: array bool{facet}, group_name: str, slot=1) -> None.
    Set facet group from an array.
    Where membership True set the corresponding facet to be a member of group group_name in the given slot.
    """
    pass




from typing import Any

def branch(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact branch vectors in the global coordinate system.
    """
    pass

def extra(*args, **kwargs) -> Any:
    """
    (slot: int) -> array float{contact} or float{contact}.
    Get the contact extra data in the given slot as an array.
    Extra variables accessed by array must be of type float or vec.
    """
    pass

def fill_branch(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact branch vectors in the global coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_force_global(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact force in the global coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_force_local(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact force in the local coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_force_normal(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact normal force.
    The array must be the correct shape.
    """
    pass

def fill_force_shear(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact shear force magnitude.
    The array must be the correct shape.
    """
    pass

def fill_gap(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact gap.
    The array must be the correct shape.
    """
    pass

def fill_moment1_global(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact moment acting on end 1 in the global coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_moment1_local(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact moment acting on end 1 in the local coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_moment2_global(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact moment acting on end 2 in the global coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_moment2_local(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact moment acting on end 2 in the local coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_normal(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact normal.
    The array must be the correct shape.
    """
    pass

def fill_offset(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact offset.
    The array must be the correct shape.
    """
    pass

def fill_pos(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact position.
    The array must be the correct shape.
    """
    pass

def force_global(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact force in the global coordinate system.
    """
    pass

def force_local(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact force in the local coordinate system.
    """
    pass

def force_normal(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact normal force.
    """
    pass

def force_shear(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact shear force magnitude.
    """
    pass

def gap(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact gap.
    """
    pass

def ids(*args, **kwargs) -> Any:
    """
    () -> array int{contact}.
    Get the contact ids as an array.
    """
    pass

def in_group(*args, **kwargs) -> Any:
    """
    (group_name: str, slot=1) -> array bool{contact}.
    Return contact group membership as a Boolean array.
    """
    pass

def moment1_global(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact moment acting on end 1 in the global coordinate system.
    """
    pass

def moment1_local(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact moment acting on end 1 in the local coordinate system.
    """
    pass

def moment2_global(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact moment acting on end 2 in the global coordinate system.
    """
    pass

def moment2_local(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact moment acting on end 2 in the local coordinate system.
    """
    pass

def normal(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact normal.
    """
    pass

def offset(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact offset.
    """
    pass

def pos(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact position.
    """
    pass

def set_extra(*args, **kwargs) -> Any:
    """
    (slot: int, data: array float{contact} or float{contact}) -> None.
    Set the contact extra data in the given slot with an array.
    Extra variables set by array must be of type float or vec.
    """
    pass

def set_group(*args, **kwargs) -> Any:
    """
    (membership: array bool{contact}, group_name: str, slot=1) -> None.
    Set contact group from an array.
    Where membership True set the corresponding contact to be a member of group group_name in the given slot.
    """
    pass




from typing import Any

def extra(*args, **kwargs) -> Any:
    """
    (slot: int) -> array float{pebble} or float{pebble}.
    Get the pebble extra data in the given slot as an array.
    Extra variables accessed by array must be of type float or vec.
    """
    pass

def fill_pos(*args, **kwargs) -> Any:
    """
    (data: array float{pebble,2}) -> None.
    Fill an existing array with the pebble location.
    The array must be the correct shape.
    """
    pass

def fill_radius(*args, **kwargs) -> Any:
    """
    (data: array float{ball}) -> None.
    Fill an existing array with the pebble radii.
    The array must be the correct shape.
    """
    pass

def fill_vel(*args, **kwargs) -> Any:
    """
    (data: array float{pebble,2}) -> None.
    Fill an existing array with the pebble velocity.
    The array must be the correct shape.
    """
    pass

def ids(*args, **kwargs) -> Any:
    """
    () -> array int{pebble}.
    Get the pebble ids as an array.
    """
    pass

def in_group(*args, **kwargs) -> Any:
    """
    (group_name: str, slot=1) -> array bool{pebble}.
    Return pebble group membership as a Boolean array.
    """
    pass

def pos(*args, **kwargs) -> Any:
    """
    () -> array float{pebble,2}.
    Get a numpy array of the pebble location.
    """
    pass

def radius(*args, **kwargs) -> Any:
    """
    () -> array float{ball}.
    Get a numpy array of the pebble radii.
    """
    pass

def set_extra(*args, **kwargs) -> Any:
    """
    (slot: int, data: array float{pebble} or float{pebble}) -> None.
    Set the pebble extra data in the given slot with an array.
    Extra variables set by array must be of type float or vec.
    """
    pass

def set_group(*args, **kwargs) -> Any:
    """
    (membership: array bool{pebble}, group_name: str, slot=1) -> None.
    Set pebble group from an array.
    Where membership True set the corresponding pebble to be a member of group group_name in the given slot.
    """
    pass

def set_pos(*args, **kwargs) -> Any:
    """
    (data: array float{pebble,2}) -> None.
    Set the pebble location from an array.
    """
    pass

def set_radius(*args, **kwargs) -> Any:
    """
    (data: array float{ball}) -> None.
    Set the pebble radii from an array.
    """
    pass

def vel(*args, **kwargs) -> Any:
    """
    () -> array float{pebble,2}.
    Get a numpy array of the pebble velocity.
    """
    pass




import itasca
from typing import Union

# There may be many contacts, however, I only use the following two contacts.
# You can artificially specify contact types like the following code:
# ```
# for contact in itasca.contact.list():
#     contact: Union[itasca.BallFacetContact, itasca.BallBallContact]
# ```
Contacts = Union[itasca.BallBallContact, itasca.BallFacetContact]



from typing import Any, Tuple

from . import template
from . import pebble
from . import thermal

def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid Clump objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass

def count(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the number of clumps.
    """
    pass

def energies(*args, **kwargs) -> Any:
    """
    () -> dict {str: float}.
    Get the clump total energy contribution as a dictionary with string keys and float values.
    """
    pass

def energy(*args, **kwargs) -> Any:
    """
    (energy_name: str) -> float.
    Get the clump total energy contribution.
    This is the sum over all clumps of the energy with name energy_na,e (see the set energy command).
    Available energies are: "energy-body", "energy-damp" and "energy-kinetic".
    """
    pass

def find(*args, **kwargs) -> Any:
    """
    (id: int) -> Clump object.
    Get the Clump object with the given ID number.
    """
    pass

def inbox(*args, **kwargs) -> Any:
    """
    (lower_bound: vec, upper_bound: vec, intersect=True) -> Tuple of Clump objects.
    Get a tuple of clumps with extents intersecting a box.
    The extent is the axis-aligned bounding box of the clump.
    The if the optional keyword argument intersect is False only clumps with positions falling in the box are returned.
    """
    pass

def list(*args, **kwargs) -> Any:
    """
    () -> Clump object iterator.
    Get a clump iterator object.
    """
    pass

def maxid(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the maximum clump ID.
    """
    pass

def near(*args, **kwargs) -> Any:
    """
    (point: vec, radius=0.0) -> Clump object.
    Find the closest clump to a point.
    If the optional keyword argument radius is non-zero, only search within this radius.
    """
    pass

class Clump:
    __hash__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def add_pebble(self, *args, **kwargs) -> Any:
        """
        (pebble_radius: float, pebble_centroid: vec [,id: int]) -> Pebble object.
        Add a pebble to a clump.
        This operation has no impact on the clump inertial attributes.
        The clump no longer refers to a clump template.
        The pebble must fall within the model domain.
        """
        pass
    
    def calculate(self, *args, **kwargs) -> Any:
        """
        (error_percentage=0.01) -> None.
        Calculate the clump interial properties.
        The optional error_percentage argument should be in the range 0.0005 < x < 1.
        The current pebble distribution is used with a voxelation approach to calculate the interial properties within a specified error.
        See the clump template command for further details.
        The pebbles are not translated but the clump centroid location may be modified.
        The moment of inertia will be modified by changes in volume or density after this operation.
        """
        pass
    
    def contact_count(self, *args, **kwargs) -> Any:
        """
        (all=False, type=None) -> int.
        Get the number of contacts associated with this clump.
        If the optional keyword argument type is given, the count is limited to the contact types specified.
        The type keyword argument should be a Python type object (one of: itasca.BallBallContact, itasca.BallPebbleContact, itasca.PebblePebbleContact, itasca.BallFacetContact or itasca.PebbleFacetContact).
        If the (optional) keyword argument all is True the count includes virtual contacts.
        """
        pass
    
    def contacts(self, *args, **kwargs) -> Any:
        """
        ([piece: object], all=False, type=None) -> Tuple of Contact objects.
        Get a tuple of contacts associated with this clump.
        An optional argument can be included which is a piece object (a Ball, a Pebble or a Facet).
        If the optional keyword argument type is given, the returned list is limited to the contact types specified.
        The type keyword argument should be a Python type object (one of: itasca.BallBallContact, itasca.BallPebbleContact, itasca.PebblePebbleContact, itasca.BallFacetContact or itasca.PebbleFacetContact).
        If the (optional) keyword argument all is True the returned list includes virtual contacts.
        .
        """
        pass
    
    def damp(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump local damping.
        """
        pass
    
    def delete(self, *args, **kwargs) -> Any:
        """
        () -> None.
        Delete this clump.
        """
        pass
    
    def delete_pebble(self, *args, **kwargs) -> Any:
        """
        (pebble: Pebble object) -> None.
        Delete a pebble from this clump.
        This operation has no impact on the clump inertial attributes.
        The clump no longer refers to a clump template.
        """
        pass
    
    def density(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump density.
        """
        pass
    
    def disp(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the clump displacement (vector).
        """
        pass
    
    def disp_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the clump displacement.
        """
        pass
    
    def disp_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the clump displacement.
        """
        pass
    
    def extra(self, *args, **kwargs) -> Any:
        """
        (slot: int) -> any.
        Get the clump extra data in the given slot.
        """
        pass
    
    def fix(self, *args, **kwargs) -> Any:
        """
        (component: int) -> bool.
        Get the clump fixity condition.
        The integer degree-of-freedom cooresponds to the following order: in 3D: 0) x-velocity, 1) y-velocity, 2) z-velocity, 3) x-angular velocity, 4) y-angular velocity, 5) z-angular velocity.
        in 2D: 0) x-velocity, 1) y-velocity, 2) angular velocity.
        The return value is false for free and true for fixed conditions.
        """
        pass
    
    def force_app(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the clump applied force (vector).
        """
        pass
    
    def force_app_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the clump applied force.
        """
        pass
    
    def force_app_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the clump applied force.
        """
        pass
    
    def force_contact(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the clump contact force (vector).
        """
        pass
    
    def force_contact_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the clump contact force.
        """
        pass
    
    def force_contact_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the clump contact force.
        """
        pass
    
    def force_unbal(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the clump unbalanced force (vector).
        """
        pass
    
    def force_unbal_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the clump unbalanced force.
        """
        pass
    
    def force_unbal_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the clump unbalanced force.
        """
        pass
    
    def fragment(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the clump fragment ID.
        """
        pass
    
    def group(self, *args, **kwargs) -> Any:
        """
        ([slot: str]) -> str.
        Get the clump group name in a given slot.
        """
        pass
    
    def group_remove(self, *args, **kwargs) -> Any:
        """
        (group_name: str) -> bool.
        Remove from the given group from all group slots of the clump.
        One argument of type string, giving the group name, is required.
        The return value is a bool which is True if the group was removed from any slot, otherwise False.
        """
        pass
    
    def groups(self, *args, **kwargs) -> Any:
        """
        () -> {slot: group_name}.
        Get a dictionary describing which groups this clump is part of.
        The keys of the dictionary are the slot names and the values are the group names.
        """
        pass
    
    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the clump id.
        """
        pass
    
    def in_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str[, slot: str]) -> bool.
        Test if the clump is part of a given group.
        If the optional argument slot is given, only that slot is searched.
        Otherwise, all group slots are searched.
        """
        pass
    
    def mass(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump inertial mass.
        """
        pass
    
    def mass_real(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump real (gravitational) mass.
        """
        pass
    
    def moi(self, *args, **kwargs) -> Any:
        """
        () -> tensor.
        Get the clump moment of intertia.
        In 2D the polar moment of intertial is used so the return value is a float.
        """
        pass
    
    def moi_fix(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Get the moment of intertia fixity state.
        When activated, the clump moment of inertia will not be automatically updated when a clump is scaled, the volume is changed, or when the density is changed.
        This is activated automatically when either the volume or moment of inertia is set manually via the clump attribute command or via the Clump object methods vol(), moi_real() or moi_prinreal().
        """
        pass
    
    def moi_prin_real(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the clump real principal moment of inertia.
        When modified no other clump attributes are changed (e.g., pebble sizes, clump volume, etc.).
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a clump unless the user changes the fix state (see the Clump moi_fix() method) (vector).
        """
        pass
    
    def moi_prin_real_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the clump real principal moment of inertia.
        When modified no other clump attributes are changed (e.g., pebble sizes, clump volume, etc.).
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a clump unless the user changes the fix state (see the Clump moi_fix() method).
        """
        pass
    
    def moi_prin_real_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the clump real principal moment of inertia.
        When modified no other clump attributes are changed (e.g., pebble sizes, clump volume, etc.).
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a clump unless the user changes the fix state (see the Clump moi_fix() method).
        """
        pass
    
    def moi_real(self, *args, **kwargs) -> Any:
        """
        () -> tensor.
        Get the clump real moment of intertia.
        In 2D the polar moment of intertial is used so the return value is a float.
        """
        pass
    
    def moment_app(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump applied moment.
        """
        pass
    
    def moment_contact(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump contact moment.
        """
        pass
    
    def moment_unbal(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump unbalanced moment.
        """
        pass
    
    def pebbles(self, *args, **kwargs) -> Any:
        """
        () -> Pebble iterator object.
        Get the pebbles of this clump.
        """
        pass
    
    def pos(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the clump centroid location (vector).
        """
        pass
    
    def pos_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the clump centroid location.
        """
        pass
    
    def pos_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the clump centroid location.
        """
        pass
    
    def rotate(self, *args, **kwargs) -> Any:
        """
        (axis: vec, rotation_angle: float) -> None.
        Rotate a clump.
        The rotation point is the clump position and it is rotated in a right handed sense about the given axis by rotation_angle degrees.
        """
        pass
    
    def rotation(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump orientation.
        """
        pass
    
    def scale_sphere(self, *args, **kwargs) -> Any:
        """
        (diameter: float) -> None.
        Scale the clump to an equivalent sphere.
        The clump is scaled so that its {area in 2D; volume in 3D} is the same as a {circle in 2D; sphere in 3D} of the given diameter.
        The pebbles are scaled, their positions are modified and the volume is updated.
        All pebbles of the scaled clump must fall within the model domain for the operation to succeed.
        If the volume or moment of inertia has been previously specified by the user either via clump attribute or via the Clump object methods (vol(), moi_prin_real() or moi_real()), the principal moment of inertial is in a fixed state and will not be scaled.
        See the Cump object moi_fix() method for further details.
        """
        pass
    
    def scale_vol(self, *args, **kwargs) -> Any:
        """
        (volume: float) -> None.
        Scale the clump.
        The clump is scaled so that its {volume per unit thickness in 2D; volume in 3D} is as specified.
        The pebbles are scaled, their positions are modified and the volume is updated.
        All pebbles of the scaled clump must fall within the model domain for the operation to succeed.
        If the volume or moment of inertia has been previously specified by the user either via clump attribute or via the Clump object methods (vol(), moi_prin_real() or moi_real()), the principal moment of inertial is in a fixed state and will not be scaled.
        See the Cump object moi_fix() method for further details.
        """
        pass
    
    def set_damp(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the clump local damping.
        """
        pass
    
    def set_density(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the clump density.
        """
        pass
    
    def set_disp(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the clump displacement (vector).
        """
        pass
    
    def set_disp_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the clump displacement.
        """
        pass
    
    def set_disp_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the clump displacement.
        """
        pass
    
    def set_extra(self, *args, **kwargs) -> Any:
        """
        (slot: int, value: any) -> None.
        Set the clump extra data in the given slot.
        """
        pass
    
    def set_fix(self, *args, **kwargs) -> Any:
        """
        (component: int, fixity: bool) -> None.
        Get the clump fixity condition.
        The integer degree-of-freedom cooresponds to the following order: in 3D: 1) x-velocity, 2) y-velocity, 3) z-velocity, 4) x-angular velocity, 5) y-angular velocity, 6) z-angular velocity.
        in 2D: 1) x-velocity, 2) y-velocity, 3) angular velocity.
        The fixity value is false for free and true for fixed conditions.
        """
        pass
    
    def set_force_app(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the clump applied force (vector).
        """
        pass
    
    def set_force_app_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the clump applied force.
        """
        pass
    
    def set_force_app_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the clump applied force.
        """
        pass
    
    def set_force_contact(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the clump contact force (vector).
        """
        pass
    
    def set_force_contact_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the clump contact force.
        """
        pass
    
    def set_force_contact_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the clump contact force.
        """
        pass
    
    def set_fragment(self, *args, **kwargs) -> Any:
        """
        (id: int) -> None.
        Set clump fragment ID.
        """
        pass
    
    def set_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str[, slot: str]) -> None.
        Set the clump group name in a given slot.
        """
        pass
    
    def set_moi_fix(self, *args, **kwargs) -> Any:
        """
        (fixity: bool) -> None.
        Set the moment of intertia fixity state.
          When activated, the clump moment of inertia will not be automatically updated when a clump is scaled, the volume is changed, or when the density is changed.
        This is activated automatically when either the volume or moment of inertia is set manually via the clump attribute command or via the Clump object methods vol(), moi_real() or moi_prinreal().
        """
        pass
    
    def set_moi_prin_real(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the clump real principal moment of inertia.
        When modified no other clump attributes are changed (e.g., pebble sizes, clump volume, etc.).
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a clump unless the user changes the fix state (see the Clump moi_fix() method) (vector).
        """
        pass
    
    def set_moi_prin_real_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the clump real principal moment of inertia.
        When modified no other clump attributes are changed (e.g., pebble sizes, clump volume, etc.).
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a clump unless the user changes the fix state (see the Clump moi_fix() method).
        """
        pass
    
    def set_moi_prin_real_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the clump real principal moment of inertia.
        When modified no other clump attributes are changed (e.g., pebble sizes, clump volume, etc.).
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a clump unless the user changes the fix state (see the Clump moi_fix() method).
        """
        pass
    
    def set_moi_real(self, *args, **kwargs) -> Any:
        """
        () -> tensor.
        Get the clump real moment of intertia.
        In 2D the polar moment of intertial is used so the return value is a float.
        When modified no other clump attributes are changed (e.g., pebble sizes, clump volume, etc.).
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a clump unless the user changes the fix state (see the Clump moi_fix() method).
        """
        pass
    
    def set_moment_app(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the clump applied moment.
        """
        pass
    
    def set_moment_contact(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the clump contact moment.
        """
        pass
    
    def set_pos(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the clump centroid location (vector).
        """
        pass
    
    def set_pos_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the clump centroid location.
        """
        pass
    
    def set_pos_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the clump centroid location.
        """
        pass
    
    def set_rotation(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the clump orientation.
        """
        pass
    
    def set_spin(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the clump angular velocity.
        """
        pass
    
    def set_vel(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the clump velocity (vector).
        """
        pass
    
    def set_vel_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the clump velocity.
        """
        pass
    
    def set_vel_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the clump velocity.
        """
        pass
    
    def set_vol(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the clump volume.
        In 2D this is the volume per unit thickness.
        """
        pass
    
    def spin(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump angular velocity.
        """
        pass
    
    def template(self, *args, **kwargs) -> Any:
        """
        () -> Template object or None.
        Get the template associated with this clump.
        The result will be None if no Template is associated with this clump.
        """
        pass
    
    def template_rotation(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump template relative orientation.
        """
        pass
    
    def template_scale(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump template relative scaling factor.
        """
        pass
    
    def to_global(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> vec.
        Rotate a vector from principal system.
        The returned vector is in the global axis system.
        """
        pass
    
    def to_prin(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> vec.
        Rotate a vector to principal system.
        """
        pass
    
    def valid(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Returns True if this clump is live.
        """
        pass
    
    def vel(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the clump velocity (vector).
        """
        pass
    
    def vel_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the clump velocity.
        """
        pass
    
    def vel_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the clump velocity.
        """
        pass
    
    def vol(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the clump volume.
        In 2D this is the volume per unit thickness.
        """
        pass
    
    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass
    
    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass
    
    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass
    
    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass
    
    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass
    
    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass
    

class ClumpIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass
    
    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass
    



from typing import Any, Tuple

from . import template

def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid RBlock objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass

def count(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the number of rblocks.
    """
    pass

def energies(*args, **kwargs) -> Any:
    """
    () -> dict {str: float}.
    Get the rblock total energy contribution as a dictionary with string keys and float values.
    """
    pass

def energy(*args, **kwargs) -> Any:
    """
    (energy_name: str) -> float.
    Get the rblock total energy contribution.
    This is the sum over all rblocks of the energy with name energy_na,e (see the set energy command).
    Available energies are: "energy-body", "energy-damp" and "energy-kinetic".
    """
    pass

def find(*args, **kwargs) -> Any:
    """
    (id: int) -> RBlock object.
    Get the RBlock object with the given ID number.
    """
    pass

def inbox(*args, **kwargs) -> Any:
    """
    (lower_bound: vec, upper_bound: vec, intersect=True) -> Tuple of RBlock objects.
    Get a tuple of rblocks with extents intersecting a box.
    The extent is the axis-aligned bounding box of the rblock.
    The if the optional keyword argument intersect is False only rblocks with positions falling in the box are returned.
    """
    pass

def list(*args, **kwargs) -> Any:
    """
    () -> RBlock object iterator.
    Get a rblock iterator object.
    """
    pass

def maxid(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the maximum rblock ID.
    """
    pass

def near(*args, **kwargs) -> Any:
    """
    (point: vec, radius=0.0) -> RBlock object.
    Find the closest rblock to a point.
    If the optional keyword argument radius is non-zero, only search within this radius.
    """
    pass

class RBlock:
    __hash__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def aspect_ratio(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock aspect ratio..
        """
        pass
    
    def ball_pos(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the rblock ball radius (vector).
        """
        pass
    
    def ball_pos_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the rblock ball radius.
        """
        pass
    
    def ball_pos_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the rblock ball radius.
        """
        pass
    
    def ball_radius(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock ball radius..
        """
        pass
    
    def contact_count(self, *args, **kwargs) -> Any:
        """
        (all=False, type=None) -> int.
        Get the number of contacts associated with this rblock.
        If the optional keyword argument type is given, the count is limited to the contact types specified.
        The type keyword argument should be a Python type object (one of: itasca.BallBallContact, itasca.BallPebbleContact, itasca.PebblePebbleContact, itasca.BallFacetContact or itasca.PebbleFacetContact).
        If the (optional) keyword argument all is True the count includes virtual contacts.
        """
        pass
    
    def contacts(self, *args, **kwargs) -> Any:
        """
        ([piece: object], all=False, type=None) -> Tuple of Contact objects.
        Get a tuple of contacts associated with this rblock.
        An optional argument can be included which is a piece object (a Ball, a Pebble, an RBlock or a Facet).
        If the optional keyword argument type is given, the returned list is limited to the contact types specified.
        The type keyword argument should be a Python type object (one of: itasca.BallBallContact, itasca.BallPebbleContact, itasca.PebblePebbleContact, itasca.BallFacetContact or itasca.PebbleFacetContact).
        If the (optional) keyword argument all is True the returned list includes virtual contacts.
        .
        """
        pass
    
    def damp(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock local damping.
        """
        pass
    
    def delete(self, *args, **kwargs) -> Any:
        """
        () -> None.
        Delete this rblock.
        """
        pass
    
    def density(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock density.
        """
        pass
    
    def disp(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the rblock displacement (vector).
        """
        pass
    
    def disp_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the rblock displacement.
        """
        pass
    
    def disp_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the rblock displacement.
        """
        pass
    
    def extra(self, *args, **kwargs) -> Any:
        """
        (slot: int) -> any.
        Get the rblock extra data in the given slot.
        """
        pass
    
    def fix(self, *args, **kwargs) -> Any:
        """
        (component: int) -> bool.
        Get the rblock fixity condition.
        The integer degree-of-freedom cooresponds to the following order: in 3D: 0) x-velocity, 1) y-velocity, 2) z-velocity, 3) x-angular velocity, 4) y-angular velocity, 5) z-angular velocity.
        in 2D: 0) x-velocity, 1) y-velocity, 2) angular velocity.
        The return value is false for free and true for fixed conditions.
        """
        pass
    
    def force_app(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the rblock applied force (vector).
        """
        pass
    
    def force_app_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the rblock applied force.
        """
        pass
    
    def force_app_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the rblock applied force.
        """
        pass
    
    def force_contact(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the rblock contact force (vector).
        """
        pass
    
    def force_contact_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the rblock contact force.
        """
        pass
    
    def force_contact_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the rblock contact force.
        """
        pass
    
    def force_unbal(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the rblock unbalanced force (vector).
        """
        pass
    
    def force_unbal_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the rblock unbalanced force.
        """
        pass
    
    def force_unbal_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the rblock unbalanced force.
        """
        pass
    
    def fragment(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the rblock fragment ID.
        """
        pass
    
    def group(self, *args, **kwargs) -> Any:
        """
        ([slot: str]) -> str.
        Get the rblock group name in a given slot.
        """
        pass
    
    def group_remove(self, *args, **kwargs) -> Any:
        """
        (group_name: str) -> bool.
        Remove from the given group from all group slots of the rblock.
        One argument of type string, giving the group name, is required.
        The return value is a bool which is True if the group was removed from any slot, otherwise False.
        """
        pass
    
    def groups(self, *args, **kwargs) -> Any:
        """
        () -> {slot: group_name}.
        Get a dictionary describing which groups this rblock is part of.
        The keys of the dictionary are the slot names and the values are the group names.
        """
        pass
    
    def has_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str) -> bool.
        Returns True if the rblock has the given surface property.
        """
        pass
    
    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the rblock id.
        """
        pass
    
    def in_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str[, slot: str]) -> bool.
        Test if the rblock is part of a given group.
        If the optional argument slot is given, only that slot is searched.
        Otherwise, all group slots are searched.
        """
        pass
    
    def mass(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock inertial mass.
        """
        pass
    
    def mass_real(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock real (gravitational) mass.
        """
        pass
    
    def moi(self, *args, **kwargs) -> Any:
        """
        () -> tensor.
        Get the rblock moment of intertia.
        In 2D the polar moment of intertial is used so the return value is a float.
        """
        pass
    
    def moi_fix(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Get the moment of intertia fixity state.
        When activated, the rblock moment of inertia will not be automatically updated when a rblock is scaled, the volume is changed, or when the density is changed.
        This is activated automatically when either the volume or moment of inertia is set manually via the rblock attribute command or via the RBlock object methods vol(), moi_real() or moi_prinreal().
        """
        pass
    
    def moi_prin_real(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the rblock real principal moment of inertia.
        When modified no other rblock attributes are changed.
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a rblock unless the user changes the fix state (see the RBlock moi_fix() method) (vector).
        """
        pass
    
    def moi_prin_real_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the rblock real principal moment of inertia.
        When modified no other rblock attributes are changed.
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a rblock unless the user changes the fix state (see the RBlock moi_fix() method).
        """
        pass
    
    def moi_prin_real_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the rblock real principal moment of inertia.
        When modified no other rblock attributes are changed.
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a rblock unless the user changes the fix state (see the RBlock moi_fix() method).
        """
        pass
    
    def moi_real(self, *args, **kwargs) -> Any:
        """
        () -> tensor.
        Get the rblock real moment of intertia.
        In 2D the polar moment of intertial is used so the return value is a float.
        """
        pass
    
    def moment_app(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock applied moment.
        """
        pass
    
    def moment_contact(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock contact moment.
        """
        pass
    
    def moment_unbal(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock unbalanced moment.
        """
        pass
    
    def pos(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the rblock centroid location (vector).
        """
        pass
    
    def pos_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the rblock centroid location.
        """
        pass
    
    def pos_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the rblock centroid location.
        """
        pass
    
    def prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str) -> any.
        Get a surface property value of this rblock.
        """
        pass
    
    def props(self, *args, **kwargs) -> Any:
        """
        () -> dict {str: any}.
        Get a dictionary of all the surface properties of this rblock.
        """
        pass
    
    def rotate(self, *args, **kwargs) -> Any:
        """
        (axis: vec, rotation_angle: float) -> None.
        Rotate a rblock.
        The rotation point is the rblock position and it is rotated in a right handed sense about the given axis by rotation_angle degrees.
        """
        pass
    
    def rotation(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock orientation.
        """
        pass
    
    def rounding(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock rounding..
        """
        pass
    
    def scale_sphere(self, *args, **kwargs) -> Any:
        """
        (diameter: float) -> None.
        Scale the rblock to an equivalent sphere.
        The rblock is scaled so that its {area in 2D; volume in 3D} is the same as a {circle in 2D; sphere in 3D} of the given diameter.
        If the volume or moment of inertia has been previously specified by the user either via rblock attribute or via the RBlock object methods (vol(), moi_prin_real() or moi_real()), the principal moment of inertial is in a fixed state and will not be scaled.
        See the Cump object moi_fix() method for further details.
        """
        pass
    
    def scale_vol(self, *args, **kwargs) -> Any:
        """
        (volume: float) -> None.
        Scale the rblock.
        The rblock is scaled so that its {volume per unit thickness in 2D; volume in 3D} is as specified.
        If the volume or moment of inertia has been previously specified by the user either via rblock attribute or via the RBlock object methods (vol(), moi_prin_real() or moi_real()), the principal moment of inertial is in a fixed state and will not be scaled.
        See the Cump object moi_fix() method for further details.
        """
        pass
    
    def set_damp(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the rblock local damping.
        """
        pass
    
    def set_density(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the rblock density.
        """
        pass
    
    def set_disp(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the rblock displacement (vector).
        """
        pass
    
    def set_disp_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the rblock displacement.
        """
        pass
    
    def set_disp_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the rblock displacement.
        """
        pass
    
    def set_extra(self, *args, **kwargs) -> Any:
        """
        (slot: int, value: any) -> None.
        Set the rblock extra data in the given slot.
        """
        pass
    
    def set_fix(self, *args, **kwargs) -> Any:
        """
        (component: int, fixity: bool) -> None.
        Get the rblock fixity condition.
        The integer degree-of-freedom cooresponds to the following order: in 3D: 1) x-velocity, 2) y-velocity, 3) z-velocity, 4) x-angular velocity, 5) y-angular velocity, 6) z-angular velocity.
        in 2D: 1) x-velocity, 2) y-velocity, 3) angular velocity.
        The fixity value is false for free and true for fixed conditions.
        """
        pass
    
    def set_force_app(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the rblock applied force (vector).
        """
        pass
    
    def set_force_app_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the rblock applied force.
        """
        pass
    
    def set_force_app_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the rblock applied force.
        """
        pass
    
    def set_force_contact(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the rblock contact force (vector).
        """
        pass
    
    def set_force_contact_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the rblock contact force.
        """
        pass
    
    def set_force_contact_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the rblock contact force.
        """
        pass
    
    def set_fragment(self, *args, **kwargs) -> Any:
        """
        (id: int) -> None.
        Set rblock fragment ID.
        """
        pass
    
    def set_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str[, slot: str]) -> None.
        Set the rblock group name in a given slot.
        """
        pass
    
    def set_moi_fix(self, *args, **kwargs) -> Any:
        """
        (fixity: bool) -> None.
        Set the moment of intertia fixity state.
          When activated, the rblock moment of inertia will not be automatically updated when a rblock is scaled, the volume is changed, or when the density is changed.
        This is activated automatically when either the volume or moment of inertia is set manually via the rblock attribute command or via the RBlock object methods vol(), moi_real() or moi_prinreal().
        """
        pass
    
    def set_moi_prin_real(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the rblock real principal moment of inertia.
        When modified no other rblock attributes are changed.
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a rblock unless the user changes the fix state (see the RBlock moi_fix() method) (vector).
        """
        pass
    
    def set_moi_prin_real_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the rblock real principal moment of inertia.
        When modified no other rblock attributes are changed.
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a rblock unless the user changes the fix state (see the RBlock moi_fix() method).
        """
        pass
    
    def set_moi_prin_real_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the rblock real principal moment of inertia.
        When modified no other rblock attributes are changed.
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a rblock unless the user changes the fix state (see the RBlock moi_fix() method).
        """
        pass
    
    def set_moi_real(self, *args, **kwargs) -> Any:
        """
        () -> tensor.
        Get the rblock real moment of intertia.
        In 2D the polar moment of intertial is used so the return value is a float.
        When modified no other rblock attributes are changed.
        The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a rblock unless the user changes the fix state (see the RBlock moi_fix() method).
        """
        pass
    
    def set_moment_app(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the rblock applied moment.
        """
        pass
    
    def set_moment_contact(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the rblock contact moment.
        """
        pass
    
    def set_pos(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the rblock centroid location (vector).
        """
        pass
    
    def set_pos_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the rblock centroid location.
        """
        pass
    
    def set_pos_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the rblock centroid location.
        """
        pass
    
    def set_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str, value: any) -> None.
        Set a surface property of this rblock.
        """
        pass
    
    def set_rotation(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the rblock orientation.
        """
        pass
    
    def set_spin(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the rblock angular velocity.
        """
        pass
    
    def set_vel(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the rblock velocity (vector).
        """
        pass
    
    def set_vel_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the rblock velocity.
        """
        pass
    
    def set_vel_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the rblock velocity.
        """
        pass
    
    def set_vol(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the rblock volume.
        In 2D this is the volume per unit thickness.
        """
        pass
    
    def spin(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock angular velocity.
        """
        pass
    
    def stress(self, *args, **kwargs) -> Any:
        """
        () -> tensor.
        Get the stress tensor arising from all contacts acting on the rblock.
        """
        pass
    
    def to_global(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> vec.
        Rotate a vector from principal system.
        The returned vector is in the global axis system.
        """
        pass
    
    def to_prin(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> vec.
        Rotate a vector to principal system.
        """
        pass
    
    def valid(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Returns True if this rblock is live.
        """
        pass
    
    def vel(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the rblock velocity (vector).
        """
        pass
    
    def vel_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the rblock velocity.
        """
        pass
    
    def vel_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the rblock velocity.
        """
        pass
    
    def vol(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the rblock volume.
        In 2D this is the volume per unit thickness.
        """
        pass
    
    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass
    
    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass
    
    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass
    
    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass
    
    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass
    
    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass
    

class RBlockIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass
    
    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass
    



from typing import Any

def branch(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact branch vectors in the global coordinate system.
    """
    pass

def extra(*args, **kwargs) -> Any:
    """
    (slot: int) -> array float{contact} or float{contact}.
    Get the contact extra data in the given slot as an array.
    Extra variables accessed by array must be of type float or vec.
    """
    pass

def fill_branch(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact branch vectors in the global coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_force_global(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact force in the global coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_force_local(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact force in the local coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_force_normal(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact normal force.
    The array must be the correct shape.
    """
    pass

def fill_force_shear(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact shear force magnitude.
    The array must be the correct shape.
    """
    pass

def fill_gap(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact gap.
    The array must be the correct shape.
    """
    pass

def fill_moment1_global(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact moment acting on end 1 in the global coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_moment1_local(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact moment acting on end 1 in the local coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_moment2_global(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact moment acting on end 2 in the global coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_moment2_local(*args, **kwargs) -> Any:
    """
    (data: array float{contact}) -> None.
    Fill an existing array with the contact moment acting on end 2 in the local coordinate system.
    The array must be the correct shape.
    """
    pass

def fill_normal(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact normal.
    The array must be the correct shape.
    """
    pass

def fill_offset(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact offset.
    The array must be the correct shape.
    """
    pass

def fill_pos(*args, **kwargs) -> Any:
    """
    (data: array float{contact,2}) -> None.
    Fill an existing array with the contact position.
    The array must be the correct shape.
    """
    pass

def force_global(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact force in the global coordinate system.
    """
    pass

def force_local(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact force in the local coordinate system.
    """
    pass

def force_normal(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact normal force.
    """
    pass

def force_shear(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact shear force magnitude.
    """
    pass

def gap(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact gap.
    """
    pass

def ids(*args, **kwargs) -> Any:
    """
    () -> array int{contact}.
    Get the contact ids as an array.
    """
    pass

def in_group(*args, **kwargs) -> Any:
    """
    (group_name: str, slot=1) -> array bool{contact}.
    Return contact group membership as a Boolean array.
    """
    pass

def indices(*args, **kwargs) -> Any:
    """
    () -> array int64{contact,2}.
    Get a numpy array of the indices of the rblocks on end1 and end2 of the contacts.
    The rblock indices are relative to the rblock arrays returned by the itasca.rblockarray module.
    """
    pass

def moment1_global(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact moment acting on end 1 in the global coordinate system.
    """
    pass

def moment1_local(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact moment acting on end 1 in the local coordinate system.
    """
    pass

def moment2_global(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact moment acting on end 2 in the global coordinate system.
    """
    pass

def moment2_local(*args, **kwargs) -> Any:
    """
    () -> array float{contact}.
    Get a numpy array of the contact moment acting on end 2 in the local coordinate system.
    """
    pass

def normal(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact normal.
    """
    pass

def offset(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact offset.
    """
    pass

def pos(*args, **kwargs) -> Any:
    """
    () -> array float{contact,2}.
    Get a numpy array of the contact position.
    """
    pass

def set_extra(*args, **kwargs) -> Any:
    """
    (slot: int, data: array float{contact} or float{contact}) -> None.
    Set the contact extra data in the given slot with an array.
    Extra variables set by array must be of type float or vec.
    """
    pass

def set_group(*args, **kwargs) -> Any:
    """
    (membership: array bool{contact}, group_name: str, slot=1) -> None.
    Set contact group from an array.
    Where membership True set the corresponding contact to be a member of group group_name in the given slot.
    """
    pass




from typing import Any

def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid vertex objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass

def count(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the number of verticess.
    """
    pass

def find(*args, **kwargs) -> Any:
    """
    (id: int) -> Vertex object.
    Get the vertex object with the given ID number.
    """
    pass

def maxid(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the maximum vertex ID.
    """
    pass

class Vertex:
    __hash__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the vertex id.
        """
        pass
    
    def pos(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the vertex location (vector).
        """
        pass
    
    def pos_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the vertex location.
        """
        pass
    
    def pos_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the vertex location.
        """
        pass
    
    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass
    
    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass
    
    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass
    
    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass
    
    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass
    
    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass
    

class VertexIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass
    
    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass
    



from typing import Any

def damp(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the clump local damping.
    """
    pass

def density(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the clump density.
    """
    pass

def disp(*args, **kwargs) -> Any:
    """
    () -> array float{clump,2}.
    Get a numpy array of the clump displacement.
    """
    pass

def extra(*args, **kwargs) -> Any:
    """
    (slot: int) -> array float{clump} or float{clump}.
    Get the clump extra data in the given slot as an array.
    Extra variables accessed by array must be of type float or vec.
    """
    pass

def fill_damp(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the clump local damping.
    The array must be the correct shape.
    """
    pass

def fill_density(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the clump density.
    The array must be the correct shape.
    """
    pass

def fill_disp(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Fill an existing array with the clump displacement.
    The array must be the correct shape.
    """
    pass

def fill_force_app(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Fill an existing array with the clump applied force.
    The array must be the correct shape.
    """
    pass

def fill_force_contact(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Fill an existing array with the clump contact force.
    The array must be the correct shape.
    """
    pass

def fill_force_unbal(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Fill an existing array with the clump unbalanced force.
    The array must be the correct shape.
    """
    pass

def fill_mass(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the inertial clump mass.
    The array must be the correct shape.
    """
    pass

def fill_mass_real(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the real (gravitational) clump mass.
    The array must be the correct shape.
    """
    pass

def fill_moi_prin_real(*args, **kwargs) -> Any:
    """
    (data: array float{clump,dim}) -> None.
    Fill an existing array with the clump principal moments of inertia.
    The array must be the correct shape.
    """
    pass

def fill_moment_app(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the clump applied moment.
    The array must be the correct shape.
    """
    pass

def fill_moment_contact(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the clump contact moment.
    The array must be the correct shape.
    """
    pass

def fill_moment_unbal(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the clump unbalanced moment.
    The array must be the correct shape.
    """
    pass

def fill_pos(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Fill an existing array with the clump centroid location.
    The array must be the correct shape.
    """
    pass

def fill_rotation(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the clump orientation.
    The array must be the correct shape.
    """
    pass

def fill_spin(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the clump angular velocity.
    The array must be the correct shape.
    """
    pass

def fill_vel(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Fill an existing array with the clump velocity.
    The array must be the correct shape.
    """
    pass

def fill_vol(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Fill an existing array with the clump volume.
    The array must be the correct shape.
    """
    pass

def force_app(*args, **kwargs) -> Any:
    """
    () -> array float{clump,2}.
    Get a numpy array of the clump applied force.
    """
    pass

def force_contact(*args, **kwargs) -> Any:
    """
    () -> array float{clump,2}.
    Get a numpy array of the clump contact force.
    """
    pass

def force_unbal(*args, **kwargs) -> Any:
    """
    () -> array float{clump,2}.
    Get a numpy array of the clump unbalanced force.
    """
    pass

def ids(*args, **kwargs) -> Any:
    """
    () -> array int{clump}.
    Get the clump ids as an array.
    """
    pass

def in_group(*args, **kwargs) -> Any:
    """
    (group_name: str, slot=1) -> array bool{clump}.
    Return clump group membership as a Boolean array.
    """
    pass

def mass(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the inertial clump mass.
    """
    pass

def mass_real(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the real (gravitational) clump mass.
    """
    pass

def moi_prin_real(*args, **kwargs) -> Any:
    """
    () -> array float{clump,dim}.
    Get a numpy array of the clump principal moments of inertia.
    """
    pass

def moment_app(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the clump applied moment.
    """
    pass

def moment_contact(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the clump contact moment.
    """
    pass

def moment_unbal(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the clump unbalanced moment.
    """
    pass

def pos(*args, **kwargs) -> Any:
    """
    () -> array float{clump,2}.
    Get a numpy array of the clump centroid location.
    """
    pass

def rotation(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the clump orientation.
    """
    pass

def set_damp(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Set the clump local damping from an array.
    """
    pass

def set_density(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Set the clump density from an array.
    """
    pass

def set_disp(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Set the clump displacement from an array.
    """
    pass

def set_extra(*args, **kwargs) -> Any:
    """
    (slot: int, data: array float{clump} or float{clump}) -> None.
    Set the clump extra data in the given slot with an array.
    Extra variables set by array must be of type float or vec.
    """
    pass

def set_force_app(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Set the clump applied force from an array.
    """
    pass

def set_force_contact(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Set the clump contact force from an array.
    """
    pass

def set_group(*args, **kwargs) -> Any:
    """
    (membership: array bool{clump}, group_name: str, slot=1) -> None.
    Set clump group from an array.
    Where membership True set the corresponding clump to be a member of group group_name in the given slot.
    """
    pass

def set_moi_prin_real(*args, **kwargs) -> Any:
    """
    (data: array float{clump,dim}) -> None.
    Set the clump principal moments of inertia from an array.
    When modified no other clump attributes are changed (e.g., pebble sizes, clump volume, etc.).
    The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a clump unless the user changes the fix state.
    """
    pass

def set_moment_app(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Set the clump applied moment from an array.
    """
    pass

def set_moment_contact(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Set the clump contact moment from an array.
    """
    pass

def set_pos(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Set the clump centroid location from an array.
    """
    pass

def set_rotation(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Set the clump orientation from an array.
    """
    pass

def set_spin(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Set the clump angular velocity from an array.
    """
    pass

def set_vel(*args, **kwargs) -> Any:
    """
    (data: array float{clump,2}) -> None.
    Set the clump velocity from an array.
    """
    pass

def set_vol(*args, **kwargs) -> Any:
    """
    (data: array float{clump}) -> None.
    Set the clump volume from an array.
    In 2D this is the volume per unit thickness.
    When modified no other clump attributes are changed (e.g., moment of inertia, pebble sizes, etc.).
    The specification of the volume in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a clump unless the user changes the fix state.
    """
    pass

def spin(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the clump angular velocity.
    """
    pass

def vel(*args, **kwargs) -> Any:
    """
    () -> array float{clump,2}.
    Get a numpy array of the clump velocity.
    """
    pass

def vol(*args, **kwargs) -> Any:
    """
    () -> array float{clump}.
    Get a numpy array of the clump volume.
    """
    pass




from typing import Any

def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid template objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass

def count(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the number of templates.
    """
    pass

def find(*args, **kwargs) -> Any:
    """
    (id: int) -> template object.
    Get the template object with the given ID number.
    """
    pass

def list(*args, **kwargs) -> Any:
    """
    () -> template iterator object.
    Get a template iterator object.
    """
    pass

def maxid(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the maximum template ID.
    """
    pass

class Template:
    __hash__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def dipmax(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the template maximum dip in degrees.
        """
        pass
    
    def dipmin(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the template minimum dip in degrees.
        """
        pass
    
    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the template id.
        """
        pass
    
    def name(self, *args, **kwargs) -> Any:
        """
        () -> string.
        Get the template name.
        """
        pass
    
    def norientparam(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the template number of orientation parameters.
        """
        pass
    
    def nposparam(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the template number of position parameters.
        """
        pass
    
    def nsizeparam(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the template number of size parameters.
        """
        pass
    
    def orientparam(self, *args, **kwargs) -> Any:
        """
        (index) -> string or float.
        Get a orientation parameter.
        The number of parameters can be assessed with the norientparam method.
        """
        pass
    
    def orienttype(self, *args, **kwargs) -> Any:
        """
        () -> string.
        Get the template orientation type name.
        Acceptable values are uniform, gauss, powerlaw, bootstrapped, fish, fisher (3D only) and dips (3D only).
        """
        pass
    
    def posparam(self, *args, **kwargs) -> Any:
        """
        (index) -> string or float.
        Get a position parameter.
        The number of parameters can be assessed with the nposparam method.
        """
        pass
    
    def postype(self, *args, **kwargs) -> Any:
        """
        () -> string.
        Get the template orientation type name.
        Acceptable values are uniform, gauss, bootstrapped and fish.
        """
        pass
    
    def set_dipmax(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the template maximum dip in degrees.
        """
        pass
    
    def set_dipmin(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the template minimum dip in degrees.
        """
        pass
    
    def set_orientparam(self, *args, **kwargs) -> Any:
        """
        (index, parameter) -> None.
        Set an orientation parameter.
        The parameter can be either a string or a float depending on the parameter.
        The number of parameters can be assessed with the norientparam method.
        """
        pass
    
    def set_orienttype(self, *args, **kwargs) -> Any:
        """
        (value: string) -> None.
        Set the template orientation type name.
        Acceptable values are uniform, gauss, powerlaw, bootstrapped, fish, fisher (3D only) and dips (3D only).
        """
        pass
    
    def set_posparam(self, *args, **kwargs) -> Any:
        """
        (index, parameter) -> None.
        Set a position parameter.
        The parameter can be either a string or a float depending on the parameter.
        The number of parameters can be assessed with the nposparam method.
        """
        pass
    
    def set_postype(self, *args, **kwargs) -> Any:
        """
        (value: string) -> None.
        Set the template orientation type name.
        Acceptable values are uniform, gauss, bootstrapped and fish.
        """
        pass
    
    def set_sizemax(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the template maximum size.
        The fracture size is defined as the {fracture length in 2D; disk diameter in 3D}.
        """
        pass
    
    def set_sizemin(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the template minimum size.
        The fracture size is defined as the {fracture length in 2D; disk diameter in 3D}.
        """
        pass
    
    def set_sizeparam(self, *args, **kwargs) -> Any:
        """
        (index, parameter) -> None.
        Set a size parameter.
        The parameter can be either a string or a float depending on the parameter.
        The number of parameters can be assessed with the nposparam method.
        """
        pass
    
    def set_sizetype(self, *args, **kwargs) -> Any:
        """
        (value: string) -> None.
        Set the template size type name.
        Acceptable values are uniform, gauss, powerlaw, bootstrapped and fish.
        """
        pass
    
    def sizemax(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the template maximum size.
        The fracture size is defined as the {fracture length in 2D; disk diameter in 3D}.
        """
        pass
    
    def sizemin(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the template minimum size.
        The fracture size is defined as the {fracture length in 2D; disk diameter in 3D}.
        """
        pass
    
    def sizeparam(self, *args, **kwargs) -> Any:
        """
        (index) -> string or float.
        Get a size parameter.
        The number of parameters can be assessed with the nsizeparam method.
        """
        pass
    
    def sizetype(self, *args, **kwargs) -> Any:
        """
        () -> string.
        Get the template size type name.
        Acceptable values are uniform, gauss, powerlaw, bootstrapped and fish.
        """
        pass
    
    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass
    
    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass
    
    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass
    
    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass
    
    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass
    
    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass
    

class templateIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass
    
    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass
    



from typing import Any

def call_function(*args, **kwargs) -> Any:
    """
    (func: string, [(tuple: any)]) -> Any.
    Call FISH function func return result.
    """
    pass

def get(*args, **kwargs) -> Any:
    """
    (var: string) -> any.
    Return value of the FISH variable var.
    """
    pass

def has(*args, **kwargs) -> Any:
    """
    (var: string) -> Boolean.
    Return True if the FISH variable var is defined, False otherwise.
    """
    pass

def is_function(*args, **kwargs) -> Any:
    """
    (var: string) -> Boolean.
    Return True if the FISH variable var is defined and is a function, False otherwise.
    """
    pass

def set(*args, **kwargs) -> Any:
    """
    (var: string, value:any) -> None.
    Set the FISH variable var to value.
    If no FISH variable with name var exists, then it is created.
    """
    pass




from typing import Any, Tuple

from . import pebble

def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid thermal clump objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass

def count(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the number of thermal clumps.
    """
    pass

def find(*args, **kwargs) -> Any:
    """
    (id: int) -> Thermal clump object.
    Get the thermal clump object with the given ID number.
    """
    pass

def inbox(*args, **kwargs) -> Any:
    """
    (lower_bound: vec, upper_bound: vec, intersect=True) -> Tuple of thermal clump objects.
    Get a tuple of thermal clumps with extents intersecting a box.
    The extent is the axis-aligned bounding box of the clump.
    The if the optional keyword argument intersect is False then only thermal clumps with positions falling in the box are returned.
    """
    pass

def list(*args, **kwargs) -> Any:
    """
    () -> Thermal clump iterator object.
    Get a thermal clump iterator object.
    """
    pass

def maxid(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the thermal maximum clump ID.
    """
    pass

def near(*args, **kwargs) -> Any:
    """
    (point: vec, radius=0.0) -> Thermal clump object.
    Find the closest thermal clump to a point.
    If the optional keyword argument radius is non-zero, only search within this radius.
    """
    pass

class ThermalClump:
    __hash__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def clump(self, *args, **kwargs) -> Any:
        """
        () -> Clump object.
        Get the clump corresponding to this thermal clump.
        """
        pass
    
    def contact_count(self, *args, **kwargs) -> Any:
        """
        (all=False, type=None) -> int.
        Get the number of contacts associated with this thermal clump.
        If the optional keyword argument type is given, the count is limited to the contact types specified.
        The optional type keyword argument should be a Python type object (one of: itasca.BallBallThermalContact, itasca.BallPebbleThermalContact, itasca.PebblePebbleThermalContact, itasca.BallFacetThermalContact or itasca.PebbleFacetThermalContact).
        If the optional keyword argument all is True the count includes virtual contacts.
        """
        pass
    
    def contacts(self, *args, **kwargs) -> Any:
        """
        ([piece], all=False, type=None) -> tuple of Contact objects.
        Get a tuple of contacts associated with this clump.
        An optional argument can be included which is a piece object (a thermal ball, a thermal pebble or a thermal facet).
        If the optional keyword argument type is given, the returned list is limited to the contact types specified.
        The type keyword argument should be a Python type object (one of: itasca.BallBallThermalContact, itasca.BallPebbleThermalContact, itasca.PebblePebbleThermalContact, itasca.BallFacetThermalContact or itasca.PebbleFacetThermalContact).
        If the (optional) keyword argument all is True the returned list includes virtual contacts.
        """
        pass
    
    def del_temp(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the thermal clump temperature increment.
        """
        pass
    
    def expansion(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the thermal clump expansion coefficient.
        """
        pass
    
    def extra(self, *args, **kwargs) -> Any:
        """
        (slot: int) -> any.
        Get the thermal clump extra data in the given slot.
        """
        pass
    
    def fix(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Get the thermal clump fixity.
        """
        pass
    
    def group(self, *args, **kwargs) -> Any:
        """
        ([slot: int]) -> str.
        Get the thermal clump group name in a given slot.
        """
        pass
    
    def group_remove(self, *args, **kwargs) -> Any:
        """
        (group_name: str ) -> int.
        Remove from the given group from the thermal clump.
        One argument of type string, giving the group name, is required.
        The return value is an integer which is the first slot in which the group name was found or -1 if not found.
        """
        pass
    
    def groups(self, *args, **kwargs) -> Any:
        """
        () -> tuple of strings.
        Get a tuple of group names assigned to this thermal clump.
        """
        pass
    
    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the thermal clump id.
        """
        pass
    
    def in_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str) -> bool.
        Test if the thermal clump is part of a given group.
        All group slots are searched.
        """
        pass
    
    def pebbles(self, *args, **kwargs) -> Any:
        """
        () -> Thermal pebble iterator object.
        Get the thermal pebbles of this thermal clump.
        """
        pass
    
    def pos(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the thermal clump location (vector).
        """
        pass
    
    def pos_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the thermal clump location.
        """
        pass
    
    def pos_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the thermal clump location.
        """
        pass
    
    def power_app(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the thermal clump applied power.
        """
        pass
    
    def power_unbal(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the thermal clump unbalanced power.
        """
        pass
    
    def set_del_temp(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the thermal clump temperature increment.
        """
        pass
    
    def set_expansion(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the thermal clump expansion coefficient.
        """
        pass
    
    def set_extra(self, *args, **kwargs) -> Any:
        """
        (slot: int, value: any) -> None.
        Set the thermal clump extra data in the given slot.
        """
        pass
    
    def set_fix(self, *args, **kwargs) -> Any:
        """
        (value: bool) -> None.
        Set the thermal clump fixity.
        """
        pass
    
    def set_group(self, *args, **kwargs) -> Any:
        """
        ([group_name: str[, slot: int]]) -> None.
        Set the thermal clump group name in a given slot.
        """
        pass
    
    def set_pos(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the thermal clump location (vector).
        """
        pass
    
    def set_pos_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the thermal clump location.
        """
        pass
    
    def set_pos_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the thermal clump location.
        """
        pass
    
    def set_power_app(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the thermal clump applied power.
        """
        pass
    
    def set_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str, value: any) -> None.
        Set a surface property of all thermal pebbles.
        """
        pass
    
    def set_specific_heat(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the thermal clump specific heat.
        """
        pass
    
    def set_temp(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the thermal clump temperature.
        """
        pass
    
    def specific_heat(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the thermal clump specific heat.
        """
        pass
    
    def temp(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the thermal clump temperature.
        """
        pass
    
    def valid(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Returns True if this thermal clump is live.
        """
        pass
    
    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass
    
    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass
    
    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass
    
    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass
    
    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass
    
    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass
    

class ThermalClumpIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass
    
    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass
    



from typing import Any, Tuple

from . import facet

def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid thermal Wall objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass

def count(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the number of thermal walls.
    """
    pass

def find(*args, **kwargs) -> Any:
    """
    (id: int) -> Thermal wall object.
    Get the thermal wall object with the given ID number.
    """
    pass

def inbox(*args, **kwargs) -> Any:
    """
    (lower_bound: vec, upper_bound: vec, intersect=True) -> Tuple of thermal wall objects.
    Get a tuple of thermal walls with extents intersecting a box.
    The extent is the axis-aligned bounding box of the thermal wall.
    The if the optional keyword argument intersect is False then only thermal walls with positions falling in the box are returned.
    """
    pass

def list(*args, **kwargs) -> Any:
    """
    () -> Thermal wall iterator object.
    Get a thermal wall iterator object.
    """
    pass

def maxid(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the maximum thermal wall ID.
    """
    pass

def near(*args, **kwargs) -> Any:
    """
    (point: vec, radius=0.0) -> Thermal wall object.
    Find the closest wall to a point.
    If the optional keyword argument radius is non-zero, only search within this radius.
    """
    pass

class ThermalWall:
    __hash__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def contact_count(self, *args, **kwargs) -> Any:
        """
        (all=False, type=None) -> int.
        Get the number of contacts associated with this thermal wall.
        If the optional keyword argument type is given, the count is limited to the contact types specified.
        The optional type keyword argument should be a Python type object (one of: itasca.BallBallThermalContact, itasca.BallPebbleThermalContact, itasca.PebblePebbleThermalContact, itasca.BallFacetThermalContact or itasca.PebbleFacetThermalContact).
        If the optional keyword argument all is True the count includes virtual contacts.
        """
        pass
    
    def contacts(self, *args, **kwargs) -> Any:
        """
        ([piece], all=False, type=None) -> tuple of Contact objects.
        Get a tuple of contacts associated with this wall.
        An optional argument can be included which is a piece object (a thermal ball, a thermal pebble or a thermal facet).
        If the optional keyword argument type is given, the returned list is limited to the contact types specified.
        The type keyword argument should be a Python type object (one of: itasca.BallBallThermalContact, itasca.BallPebbleThermalContact, itasca.PebblePebbleThermalContact, itasca.BallFacetThermalContact or itasca.PebbleFacetThermalContact).
        If the (optional) keyword argument all is True the returned list includes virtual contacts.
        """
        pass
    
    def extra(self, *args, **kwargs) -> Any:
        """
        (slot: int) -> any.
        Get the thermal wall extra data in the given slot.
        """
        pass
    
    def facets(self, *args, **kwargs) -> Any:
        """
        () -> Thermal facet iterator object.
        Get the thermal facets of this thermal wall.
        """
        pass
    
    def group(self, *args, **kwargs) -> Any:
        """
        ([slot: int]) -> str.
        Get the thermal wall group name in a given slot.
        """
        pass
    
    def group_remove(self, *args, **kwargs) -> Any:
        """
        (group_name: str ) -> int.
        Remove from the given group from the thermal wall.
        One argument of type string, giving the group name, is required.
        The return value is an integer which is the first slot in which the group name was found or -1 if not found.
        """
        pass
    
    def groups(self, *args, **kwargs) -> Any:
        """
        () -> tuple of strings.
        Get a tuple of group names assigned to this thermal wall.
        """
        pass
    
    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the thermal wall id.
        """
        pass
    
    def in_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str) -> bool.
        Test if the thermal wall is part of a given group.
        All group slots are searched.
        """
        pass
    
    def pos(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the thermal wall location (vector).
        """
        pass
    
    def pos_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the thermal wall location.
        """
        pass
    
    def pos_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the thermal wall location.
        """
        pass
    
    def set_extra(self, *args, **kwargs) -> Any:
        """
        (slot: int, value: any) -> None.
        Set the thermal wall extra data in the given slot.
        """
        pass
    
    def set_group(self, *args, **kwargs) -> Any:
        """
        ([group_name: str[, slot: int]]) -> None.
        Set the thermal wall group name in a given slot.
        """
        pass
    
    def set_pos(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the thermal wall location (vector).
        """
        pass
    
    def set_pos_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the thermal wall location.
        """
        pass
    
    def set_pos_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the thermal wall location.
        """
        pass
    
    def set_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str, value: any) -> None.
        Set a surface property of all thermal facets.
        """
        pass
    
    def set_temp(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the thermal wall facet temperatures of this wall.
        """
        pass
    
    def valid(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Returns True if this thermal wall is live.
        """
        pass
    
    def wall(self, *args, **kwargs) -> Any:
        """
        () -> Wall object.
        Get the wall corresponding to this thermal wall.
        """
        pass
    
    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass
    
    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass
    
    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass
    
    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass
    
    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass
    
    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass
    

class ThermalWallIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass
    
    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass
    



from typing import Any, Tuple

from . import fracture
from . import template
from . import inter
from . import setinter
from . import vertex

def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid dfn objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass

def center_density(*args, **kwargs) -> Any:
    """
    (lower=domain lower bound, upper=domain upper bound) -> float.
    Get the fracture center density.
    The fracture center density is defined as the {P20 value in 2D; P30 value in 3D} as measured in the specified region.
    {P20 in 2D; P30 in 3D} is the number of fracture centers in the specified region per unit area (volume).
    All fractures from all dfns are used in this calculation.
    """
    pass

def count(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the number of dfns.
    """
    pass

def create(*args, **kwargs) -> Any:
    """
    (dfn_name=Empty) -> DFN object.
    Create an empty dfn with optional name dfn_name.
    """
    pass

def density(*args, **kwargs) -> Any:
    """
    (lower=domain lower bound, upper=domain upper bound) -> float.
    Get the fracture mass density.
    The fracture mass density is defined as the {P21 value in 2D; P32 value in 3D} as measured in the specified region.
    {P21 in 2D; P32 in 3D} is the cumulated fracture length (surface) in the specified region per unit area (volume).
    All fractures from all dfns are used in this calculation.
    """
    pass

def find(*args, **kwargs) -> Any:
    """
    (id: int or name: str) -> DFN object.
    Get the dfn object with the given ID number or name.
    """
    pass

def list(*args, **kwargs) -> Any:
    """
    () -> DFN iterator object.
    Get a dfn iterator object.
    """
    pass

def maxid(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the maximum dfn ID.
    """
    pass

def p10(*args, **kwargs) -> Any:
    """
    (pt1=origin, pt2=origin) -> float.
    Get the P10, or the fracture density per unit length, along the line from pt1 to pt2.
    All fractures from all dfns are used in this calculation.
    """
    pass

def percolation(*args, **kwargs) -> Any:
    """
    (lower=domain lower bound, upper=domain upper bound) -> float.
    Get the fracture percolation.
    The fracture percolation is measured in the region and is defined as {the sum of the squared fracture lengths per unit area in 2D; the sum of pi^2 (area/pi)^(1.5) per unit volume, where area is the fracture area in 3D}.
    """
    pass

class DFN:
    __hash__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def center_density(self, *args, **kwargs) -> Any:
        """
        (lower=domain lower bound, upper=domain upper bound) -> float.
        Get the fracture center density.
        The fracture center density is defined as the {P20 value in 2D; P30 value in 3D} as measured in the specified region.
        {P20 in 2D; P30 in 3D} is the number of fracture centers in the specified region per unit area (volume).
        """
        pass
    
    def clone_fracture(self, *args, **kwargs) -> Any:
        """
        (Fracture object) -> Fracture object.
        Clone the fracture, creating a new one in the dfn.
        The fracturecan come from any dfn.
        """
        pass
    
    def contacts(self, *args, **kwargs) -> Any:
        """
        (all=False, type=None) -> tuple of Contact objects.
        Get a tuple of contacts associated with the fractures in this dfn.
        The type keyword argument should be a Python type object (one of: itasca.BallBallContact, itasca.BallPebbleContact, itasca.PebblePebbleContact, itasca.BallFacetContact or itasca.PebbleFacetContact).
        If the (optional) keyword argument all is True the returned list includes virtual contacts.
        """
        pass
    
    def create_fracture(self, *args, **kwargs) -> Any:
        """
        (pos=origin, size=0, dip=0 ,dipdir=0 (3D ONLY), id=0, vertices=array) -> Fracture object.
        Create a fracture and addit to this dfn.
        In 3D a disk shaped fracture can be created where size is the fracture diameter, pos is the center of the fracture and dip/dipdir and the dip and dip direction in degrees, respectively.In 2D, a line segement fracture can be created where size is the fracture length, pos is the center and dip is the fracture dip in degrees.
        Otherwise one may specify a number of verticesfor {2 in 2D; 3 or more in 3D} that delineate the fracture vertices.
        """
        pass
    
    def delete(self, *args, **kwargs) -> Any:
        """
        () -> None.
        Delete this DFN.
        """
        pass
    
    def density(self, *args, **kwargs) -> Any:
        """
        (lower=domain lower bound, upper=domain upper bound) -> float.
        Get the fracture mass density.
        The fracture mass density is defined as the {P21 value in 2D; P32 value in 3D} as measured in the specified region.
        {P21 in 2D; P32 in 3D} is the cumulated fracture length (surface) in the specified region per unit area (volume).
        """
        pass
    
    def dominance(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the dfn dominance.
        """
        pass
    
    def fracture_count(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the number of fractures in this dfn.
        """
        pass
    
    def fracture_near(self, *args, **kwargs) -> Any:
        """
        (point: vec, radius=0.0) -> Fracture object.
        Find the closest fracture to a point.
        If the optional keyword argument radius is non-zero, only search within this radius.
        """
        pass
    
    def fractures(self, *args, **kwargs) -> Any:
        """
        () -> Fracture iterator object.
        Get a fracture iterator object.
        """
        pass
    
    def fractures_inbox(self, *args, **kwargs) -> Any:
        """
        (lower_bound: vec, upper_bound: vec, intersect=True) -> Tuple of fracture objects.
        Get a tuple of fractures with extents intersecting a box.
        The extent is the axis-aligned bounding box of the fracture.
        The if the optional keyword argument intersect is False only fractures entirely falling within the box are returned.
        Only fractures belonging to this dfn are are searched.
        """
        pass
    
    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the dfn id.
        """
        pass
    
    def name(self, *args, **kwargs) -> Any:
        """
        () -> string.
        Get the dfn name.
        """
        pass
    
    def p10(self, *args, **kwargs) -> Any:
        """
        (pt1=origin, pt2=origin) -> float.
        Get the P10, or the fracture density per unit length, along the line from pt1 to pt2.
        """
        pass
    
    def percolation(self, *args, **kwargs) -> Any:
        """
        (lower=domain lower bound, upper=domain upper bound) -> float.
        Get the fracture percolation.
        The fracture percolation is measured in the region and is defined as {the sum of the squared fracture lengths per unit area in 2D; the sum of pi^2 (area/pi)^(1.5) per unit volume, where area is the fracture area in 3D}.
        """
        pass
    
    def set_dominance(self, *args, **kwargs) -> Any:
        """
        (value: int) -> None.
        Set the dfn dominance.
        """
        pass
    
    def set_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str, value: any) -> None.
        Set a surface property of all fractures of this dfn.
        """
        pass
    
    def template(self, *args, **kwargs) -> Any:
        """
        () -> DFN Template object.
        Get the template used to create this dfn.
        """
        pass
    
    def valid(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Returns True if this dfn is live.
        """
        pass
    
    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass
    
    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass
    
    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass
    
    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass
    
    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass
    
    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass
    

class DFNIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass
    
    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass
    



from typing import Any, Tuple

def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid fracture objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass

def count(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the number of fractures.
    """
    pass

def find(*args, **kwargs) -> Any:
    """
    (id: int) -> Fracture object.
    Get the fracture object with the given ID number.
    """
    pass

def inbox(*args, **kwargs) -> Any:
    """
    (lower_bound: vec, upper_bound: vec, intersect=True) -> Tuple of fracture objects.
    Get a tuple of fractures with extents intersecting a box.
    The extent is the axis-aligned bounding box of the fracture.
    The if the optional keyword argument intersect is False only fractures falling entirely within the box are returned.
    All fractures are searched.
    """
    pass

def list(*args, **kwargs) -> Any:
    """
    () -> Fracture iterator object.
    Get a Fracture iterator object.
    """
    pass

def maxid(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the maximum fracture ID.
    """
    pass

def near(*args, **kwargs) -> Any:
    """
    (point: vec, radius=0.0) -> Fracture object.
    Find the closest fracture to a point.
    If the optional keyword argument radius is non-zero, only search within this radius.
    All fractures are searched.
    """
    pass

class Fracture:
    __hash__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def aperture(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the fracture aperture.
        """
        pass
    
    def contact_count(self, *args, **kwargs) -> Any:
        """
        (all=False, type=None) -> int.
        Get the number of contacts associated with this fracture.
        If the optional keyword argument type is given, the count is limited to the contact types specified.
        The optional type keyword argument should be a Python type object (one of: itasca.BallBallContact, itasca.BallPebbleContact, itasca.PebblePebbleContact, itasca.BallFacetContact or itasca.PebbleFacetContact).
        If the optional keyword argument all is True the count includes virtual contacts.
        """
        pass
    
    def contacts(self, *args, **kwargs) -> Any:
        """
        (all=False, type=None) -> tuple of Contact objects.Get a tuple of contacts associated with this fracture.
        If the optional keyword argument type is given, the returned list is limited to the contact types specified.
        The type keyword argument should be a Python type object (one of: itasca.BallBallContact, itasca.BallPebbleContact, itasca.PebblePebbleContact, itasca.BallFacetContact or itasca.PebbleFacetContact).
        If the (optional) keyword argument all is True the returned list includes virtual contacts.
        """
        pass
    
    def delete(self, *args, **kwargs) -> Any:
        """
        () -> None.
        Delete this fracture.
        """
        pass
    
    def dfn(self, *args, **kwargs) -> Any:
        """
        () -> DFN object.
        Get the dfn to which this fracture belongs.
        """
        pass
    
    def dip(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the fracture dip.
        """
        pass
    
    def extra(self, *args, **kwargs) -> Any:
        """
        (slot: int) -> any.
        Get the fracture extra data in the given slot.
        """
        pass
    
    def group(self, *args, **kwargs) -> Any:
        """
        ([slot: str]) -> str.
        Get the fracture group name in a given slot.
        """
        pass
    
    def group_remove(self, *args, **kwargs) -> Any:
        """
        (group_name: str) -> bool.
        Remove from the given group from all group slots of the fracture.
        One argument of type string, giving the group name, is required.
        The return value is a bool which is True if the group was removed from any slot, otherwise False.
        """
        pass
    
    def groups(self, *args, **kwargs) -> Any:
        """
        () -> {slot: group_name}.
        Get a dictionary describing which groups this fracture is part of.
        The keys of the dictionary are the slot names and the values are the group names.
        """
        pass
    
    def has_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str) -> bool.
        Returns True if the fracture has the given surface property.
        """
        pass
    
    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the fracture id.
        """
        pass
    
    def in_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str[, slot: str]) -> bool.
        Test if the fracture is part of a given group.
        If the optional argument slot is given, only that slot is searched.
        Otherwise, all group slots are searched.
        """
        pass
    
    def intersections(self, *args, **kwargs) -> Any:
        """
        (DFN object) -> tuple of Intersection objects.
        Get the intersections with this fracture.
        An optional DFN object can be specified to limit the intersections to fractures belonging to that dfn.
        """
        pass
    
    def intersects(self, *args, **kwargs) -> Any:
        """
        (Fracture object) -> bool.
        Get a boolean indicating the intersection status of the given fracture with this fracture.
        """
        pass
    
    def len(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the fracture length.
        """
        pass
    
    def normal(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the fracture normal (vector).
        """
        pass
    
    def normal_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the fracture normal.
        """
        pass
    
    def normal_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the fracture normal.
        """
        pass
    
    def point_near(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> vec.
        Get the point on the fracture closest to the given point (vector).
        """
        pass
    
    def pos(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the fracture location (vector).
        """
        pass
    
    def pos_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the fracture location.
        """
        pass
    
    def pos_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the fracture location.
        """
        pass
    
    def prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str) -> any.
        Get a surface property value of this fracture.
        """
        pass
    
    def props(self, *args, **kwargs) -> Any:
        """
        () -> dict {str: any}.
        Get a dictionary of all the surface properties of this fracture.
        """
        pass
    
    def set_aperture(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the fracture aperture.
        """
        pass
    
    def set_dip(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the fracture dip.
        """
        pass
    
    def set_extra(self, *args, **kwargs) -> Any:
        """
        (slot: int, value: any) -> None.
        Set the fracture extra data in the given slot.
        """
        pass
    
    def set_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str[, slot: str]) -> None.
        Set the fracture group name in a given slot.
        """
        pass
    
    def set_len(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the fracture length.
        """
        pass
    
    def set_normal(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the fracture normal (vector).
        """
        pass
    
    def set_normal_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the fracture normal.
        """
        pass
    
    def set_normal_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the fracture normal.
        """
        pass
    
    def set_pos(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the fracture location (vector).
        """
        pass
    
    def set_pos_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the fracture location.
        """
        pass
    
    def set_pos_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the fracture location.
        """
        pass
    
    def set_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str, value: any) -> None.
        Set a surface property of this fracture.
        """
        pass
    
    def valid(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Returns True if this fracture is live.
        """
        pass
    
    def vertices(self, *args, **kwargs) -> Any:
        """
        () -> tuple of Vertex objects.
        Get the vertices of this fracture.
        """
        pass
    
    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass
    
    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass
    
    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass
    
    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass
    
    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass
    
    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass
    

class FractureIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass
    
    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass
    



from typing import Any, Tuple

def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid thermal facet objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass

def count(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the number of thermal facets.
    """
    pass

def find(*args, **kwargs) -> Any:
    """
    (id: int) -> Thermal facet object.
    Get the thermal facet object with the given ID number.
    """
    pass

def inbox(*args, **kwargs) -> Any:
    """
    (lower_bound: vec, upper_bound: vec, intersect=True) -> Tuple of thermal facet objects.
    Get a tuple of thermal facets with extents intersecting a box.
    The extent is the axis-aligned bounding box of the thermal facet.
    The if the optional keyword argument intersect is False then only thermal facets with positions falling in the box are returned.
    """
    pass

def list(*args, **kwargs) -> Any:
    """
    () -> Thermal facet iterator object.
    Get a thermal facet iterator object.
    """
    pass

def maxid(*args, **kwargs) -> Any:
    """
    () -> int.
    Get the maximum thermal facet ID.
    """
    pass

def near(*args, **kwargs) -> Any:
    """
    (point: vec, radius=0.0) -> Thermal facet object.
    Find the closest thermal facet to a point.
    If the optional keyword argument radius is non-zero, only search within this radius.
    """
    pass

class ThermalFacet:
    __hash__: Any = ...
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def contact_count(self, *args, **kwargs) -> Any:
        """
        (all=False, type=None) -> int.
        Get the number of contacts associated with this thermal pebble.
        If the optional keyword argument type is given, the count is limited to the contact types specified.
        The optional type keyword argument should be a Python type object (one of: itasca.BallBallThermalContact, itasca.BallPebbleThermalContact, itasca.PebblePebbleThermalContact, itasca.BallFacetThermalContact or itasca.PebbleFacetThermalContact).
        If the optional keyword argument all is True the count includes virtual contacts.
        """
        pass
    
    def contacts(self, *args, **kwargs) -> Any:
        """
        ([piece], all=False, type=None) -> tuple of Contact objects.
        Get a tuple of contacts associated with this thermal pebble.
        An optional argument can be included which is a piece object (a thermal ball, a thermal pebble or a thermal facet).
        If the optional keyword argument type is given, the returned list is limited to the contact types specified.
        The type keyword argument should be a Python type object (one of: itasca.BallBallThermalContact, itasca.BallPebbleThermalContact, itasca.PebblePebbleThermalContact, itasca.BallFacetThermalContact or itasca.PebbleFacetThermalContact).
        If the (optional) keyword argument all is True the returned list includes virtual contacts.
        """
        pass
    
    def extra(self, *args, **kwargs) -> Any:
        """
        (slot: int) -> any.
        Get the thermal facet extra data in the given slot.
        """
        pass
    
    def facet(self, *args, **kwargs) -> Any:
        """
        () -> Pebble object.
        Get the pebble corresponding to this thermal pebble.
        """
        pass
    
    def group(self, *args, **kwargs) -> Any:
        """
        ([slot: int]) -> str.
        Get the thermal facet group name in a given slot.
        """
        pass
    
    def group_remove(self, *args, **kwargs) -> Any:
        """
        (group_name: str ) -> int.
        Remove from the given group from the thermal facet.
        One argument of type string, giving the group name, is required.
        The return value is an integer which is the first slot in which the group name was found or -1 if not found.
        """
        pass
    
    def groups(self, *args, **kwargs) -> Any:
        """
        () -> tuple of strings.
        Get a tuple of group names assigned to this thermal facet.
        """
        pass
    
    def has_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str) -> bool.
        Returns True if the thermal facet has the given surface property.
        """
        pass
    
    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the thermal facet id.
        """
        pass
    
    def in_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str) -> bool.
        Test if the thermal facet is part of a given group.
        All group slots are searched.
        """
        pass
    
    def pos(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the thermal facet location (vector).
        """
        pass
    
    def pos_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the thermal facet location.
        """
        pass
    
    def pos_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the thermal facet location.
        """
        pass
    
    def prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str) -> any.
        Get a surface property value of this thermal facet.
        """
        pass
    
    def props(self, *args, **kwargs) -> Any:
        """
        () -> dict {str: any}.
        Get a dictionary of all the surface properties of this thermal facet.
        """
        pass
    
    def set_extra(self, *args, **kwargs) -> Any:
        """
        (slot: int, value: any) -> None.
        Set the thermal facet extra data in the given slot.
        """
        pass
    
    def set_group(self, *args, **kwargs) -> Any:
        """
        ([group_name: str[, slot: int]]) -> None.
        Set the thermal facet group name in a given slot.
        """
        pass
    
    def set_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str, value: any) -> None.
        Set a surface property of this thermal facet.
        """
        pass
    
    def set_temp(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the thermal facet temperature.
        """
        pass
    
    def temp(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the thermal facet temperature.
        """
        pass
    
    def valid(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Returns True if this thermal facet is live.
        """
        pass
    
    def wall(self, *args, **kwargs) -> Any:
        """
        () -> Thermal clump object.
        Get the thermal clump corresponding to this pebble.
        """
        pass
    
    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass
    
    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass
    
    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass
    
    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass
    
    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass
    
    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass
    

class ThermalFacetIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass
    
    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass
    
    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass
    



from typing import Any, Tuple, Iterator

from . import thermal


def _plist(*args, **kwargs) -> Any:
    """
    () -> tuple of PyObject pointers for the currenly in-scope and valid Ball objects.
    This function is used for internal testing and is not needed for general PFC use.
    """
    pass


def count() -> int:
    """
    Get the number of balls.
    """
    pass


def create(*args, **kwargs) -> 'Ball':
    """
    (radius: double, centroid: vec [,id: int]) -> Ball object.
    Create a ball.
    """
    pass


def energy(energy_name: str) -> float:
    """
    Get the ball total energy contribution.
    """
    pass


def find(id: int) -> 'Ball':
    """
    Get the ball object with the given ID number.
    """
    pass


def inbox(*args, **kwargs) -> Tuple['Ball']:
    """
    (lower_bound: vec, upper_bound: vec, intersect=True) -> Tuple of ball objects.
    Get a tuple of balls with extents intersecting a box.
    The extent is the axis-aligned bounding box of the ball.
    The if the optional keyword argument intersect is False then only balls with positions falling in the box are returned.
    """
    pass


def list() -> Iterator['Ball']:
    """
    Get a ball iterator object.
    """
    pass


def maxid() -> int:
    """
    Get the maximum ball ID.
    """
    pass


def near(*args, **kwargs) -> 'Ball':
    """
    (point: vec, radius=0.0) -> Ball object.
    Find the closest ball to a point.
    If the optional keyword argument radius is non-zero, only search within this radius.
    """
    pass


class Ball:
    def __init__(self) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass

    def contact_count(self, *args, **kwargs) -> Any:
        """
        (all=False, type=None) -> int.
        Get the number of contacts associated with this ball.
        If the optional keyword argument type is given, the count is limited to the contact types specified.
        The optional type keyword argument should be a Python type object (one of: itasca.BallBallContact, itasca.BallPebbleContact, itasca.PebblePebbleContact, itasca.BallFacetContact or itasca.PebbleFacetContact).
        If the optional keyword argument all is True the count includes virtual contacts.
        """
        pass

    def contacts(self, *args, **kwargs) -> Any:
        """
        ([piece: ball], all=False, type=None) -> tuple of Contact objects.
        Get a tuple of contacts associated with this ball.
        An optional argument can be included which is a piece object (a Ball, a Pebble or a Facet).
        If the optional keyword argument type is given, the returned list is limited to the contact types specified.
        The type keyword argument should be a Python type object (one of: itasca.BallBallContact, itasca.BallPebbleContact, itasca.PebblePebbleContact, itasca.BallFacetContact or itasca.PebbleFacetContact).
        If the (optional) keyword argument all is True the returned list includes virtual contacts.
        """
        pass

    def damp(self) -> float:
        """
        Get the ball local damping.
        """
        pass

    def delete(self) -> None:
        """
        Delete this ball.
        """
        pass

    def density(self) -> float:
        """
        Get the ball density.
        """
        pass

    def disp(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the ball displacement (vector).
        """
        pass

    def disp_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the ball displacement.
        """
        pass

    def disp_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the ball displacement.
        """
        pass

    def extra(self, *args, **kwargs) -> Any:
        """
        (slot: int) -> any.
        Get the ball extra data in the given slot.
        """
        pass

    def fix(self, *args, **kwargs) -> Any:
        """
        (component: int) -> bool.
        Get the ball fixity condition.
        The first argument is the vector component.
        """
        pass

    def force_app(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the ball applied force (vector).
        """
        pass

    def force_app_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the ball applied force.
        """
        pass

    def force_app_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the ball applied force.
        """
        pass

    def force_contact(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the ball contact force (vector).
        """
        pass

    def force_contact_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the ball contact force.
        """
        pass

    def force_contact_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the ball contact force.
        """
        pass

    def force_unbal(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the ball unbalanced force (vector).
        """
        pass

    def force_unbal_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the ball unbalanced force.
        """
        pass

    def force_unbal_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the ball unbalanced force.
        """
        pass

    def fragment(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the ball fragment ID.
        """
        pass

    def group(self, *args, **kwargs) -> Any:
        """
        ([slot: str]) -> str.
        Get the ball group name in a given slot.
        """
        pass

    def group_remove(self, *args, **kwargs) -> Any:
        """
        (group_name: str) -> bool.
        Remove from the given group from all group slots of the ball.
        One argument of type string, giving the group name, is required.
        The return value is a bool which is True if the group was removed from any slot, otherwise False.
        """
        pass

    def groups(self, *args, **kwargs) -> Any:
        """
        () -> {slot: group_name}.
        Get a dictionary describing which groups this ball is part of.
        The keys of the dictionary are the slot names and the values are the group names.
        """
        pass

    def has_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str) -> bool.
        Returns True if the ball has the given surface property.
        """
        pass

    def id(self, *args, **kwargs) -> Any:
        """
        () -> int.
        Get the ball id.
        """
        pass

    def in_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str[, slot: str]) -> bool.
        Test if the ball is part of a given group.
        If the optional argument slot is given, only that slot is searched.
        Otherwise, all group slots are searched.
        """
        pass

    def mass(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball inertial mass.
        """
        pass

    def mass_real(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball real (gravitational) mass.
        """
        pass

    def moi(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball moment of inertia.
        """
        pass

    def moi_real(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball real (gravitational) moment of inertia.
        """
        pass

    def moment_app(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball applied moment.
        """
        pass

    def moment_contact(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball contact moment.
        """
        pass

    def moment_unbal(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball unbalanced moment.
        """
        pass

    def pos(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the ball centroid location (vector).
        """
        pass

    def pos_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the ball centroid location.
        """
        pass

    def pos_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the ball centroid location.
        """
        pass

    def prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str) -> any.
        Get a surface property value of this ball.
        """
        pass

    def props(self, *args, **kwargs) -> Any:
        """
        () -> dict {str: any}.
        Get a dictionary of all the surface properties of this ball.
        """
        pass

    def radius(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball radius.
        """
        pass

    def rotation(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball orientation.
        """
        pass

    def set_damp(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the ball local damping.
        """
        pass

    def set_density(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the ball density.
        """
        pass

    def set_disp(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the ball displacement (vector).
        """
        pass

    def set_disp_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the ball displacement.
        """
        pass

    def set_disp_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the ball displacement.
        """
        pass

    def set_extra(self, *args, **kwargs) -> Any:
        """
        (slot: int, value: any) -> None.
        Set the ball extra data in the given slot.
        """
        pass

    def set_fix(self, *args, **kwargs) -> Any:
        """
        (component: int, fix: bool) -> None.
        Set the ball fixity condition.
        The first argument is the vector component to set and the second components is a the fixity flag.
        """
        pass

    def set_force_app(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the ball applied force (vector).
        """
        pass

    def set_force_app_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the ball applied force.
        """
        pass

    def set_force_app_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the ball applied force.
        """
        pass

    def set_force_contact(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the ball contact force (vector).
        """
        pass

    def set_force_contact_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the ball contact force.
        """
        pass

    def set_force_contact_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the ball contact force.
        """
        pass

    def set_fragment(self, *args, **kwargs) -> Any:
        """
        (id: int) -> None.
        Set ball fragment ID.
        """
        pass

    def set_group(self, *args, **kwargs) -> Any:
        """
        (group_name: str[, slot: str]) -> None.
        Set the ball group name in a given slot.
        """
        pass

    def set_moment_app(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the ball applied moment.
        """
        pass

    def set_moment_contact(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the ball contact moment.
        """
        pass

    def set_pos(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the ball centroid location (vector).
        """
        pass

    def set_pos_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the ball centroid location.
        """
        pass

    def set_pos_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the ball centroid location.
        """
        pass

    def set_prop(self, *args, **kwargs) -> Any:
        """
        (property_name: str, value: any) -> None.
        Set a surface property of this ball.
        """
        pass

    def set_radius(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the ball radius.
        """
        pass

    def set_rotation(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the ball orientation.
        """
        pass

    def set_spin(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the ball angular velocity.
        """
        pass

    def set_vel(self, *args, **kwargs) -> Any:
        """
        (value: vec) -> None.
        Set the ball velocity (vector).
        """
        pass

    def set_vel_x(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the x-component of the ball velocity.
        """
        pass

    def set_vel_y(self, *args, **kwargs) -> Any:
        """
        (value: float) -> None.
        Set the y-component of the ball velocity.
        """
        pass

    def spin(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the ball angular velocity.
        """
        pass

    def stress(self, *args, **kwargs) -> Any:
        """
        () -> tensor.
        Get the stress tensor arising from all contacts acting on the ball.
        """
        pass

    def valid(self, *args, **kwargs) -> Any:
        """
        () -> bool.
        Returns True if this ball is live.
        """
        pass

    def vel(self, *args, **kwargs) -> Any:
        """
        () -> vec.
        Get the ball velocity (vector).
        """
        pass

    def vel_x(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the x-component of the ball velocity.
        """
        pass

    def vel_y(self, *args, **kwargs) -> Any:
        """
        () -> float.
        Get the y-component of the ball velocity.
        """
        pass

    def __eq__(self, other) -> Any:
        """
        Return self==value.
        """
        pass

    def __ge__(self, other) -> Any:
        """
        Return self>=value.
        """
        pass

    def __gt__(self, other) -> Any:
        """
        Return self>value.
        """
        pass

    def __le__(self, other) -> Any:
        """
        Return self<=value.
        """
        pass

    def __lt__(self, other) -> Any:
        """
        Return self<value.
        """
        pass

    def __ne__(self, other) -> Any:
        """
        Return self!=value.
        """
        pass


class BallIter:
    @classmethod
    def __init__(self, *args, **kwargs) -> None:
        """
        Create and return a new object.
         See help(type) for accurate signature.
        """
        pass

    def __iter__(self) -> Any:
        """
        Implement iter(self).
        """
        pass

    def __next__(self) -> Any:
        """
        Implement next(self).
        """
        pass



from typing import Any

def damp(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the rblock local damping.
    """
    pass

def density(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the rblock density.
    """
    pass

def disp(*args, **kwargs) -> Any:
    """
    () -> array float{rblock,2}.
    Get a numpy array of the rblock displacement.
    """
    pass

def extra(*args, **kwargs) -> Any:
    """
    (slot: int) -> array float{rblock} or float{rblock}.
    Get the rblock extra data in the given slot as an array.
    Extra variables accessed by array must be of type float or vec.
    """
    pass

def fill_damp(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the rblock local damping.
    The array must be the correct shape.
    """
    pass

def fill_density(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the rblock density.
    The array must be the correct shape.
    """
    pass

def fill_disp(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Fill an existing array with the rblock displacement.
    The array must be the correct shape.
    """
    pass

def fill_force_app(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Fill an existing array with the rblock applied force.
    The array must be the correct shape.
    """
    pass

def fill_force_contact(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Fill an existing array with the rblock contact force.
    The array must be the correct shape.
    """
    pass

def fill_force_unbal(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Fill an existing array with the rblock unbalanced force.
    The array must be the correct shape.
    """
    pass

def fill_mass(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the inertial rblock mass.
    The array must be the correct shape.
    """
    pass

def fill_mass_real(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the real (gravitational) rblock mass.
    The array must be the correct shape.
    """
    pass

def fill_moi_prin_real(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,dim}) -> None.
    Fill an existing array with the rblock principal moments of inertia.
    The array must be the correct shape.
    """
    pass

def fill_moment_app(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the rblock applied moment.
    The array must be the correct shape.
    """
    pass

def fill_moment_contact(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the rblock contact moment.
    The array must be the correct shape.
    """
    pass

def fill_moment_unbal(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the rblock unbalanced moment.
    The array must be the correct shape.
    """
    pass

def fill_pos(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Fill an existing array with the rblock centroid location.
    The array must be the correct shape.
    """
    pass

def fill_rotation(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the rblock orientation.
    The array must be the correct shape.
    """
    pass

def fill_spin(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the rblock angular velocity.
    The array must be the correct shape.
    """
    pass

def fill_vel(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Fill an existing array with the rblock velocity.
    The array must be the correct shape.
    """
    pass

def fill_vol(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Fill an existing array with the rblock volume.
    The array must be the correct shape.
    """
    pass

def force_app(*args, **kwargs) -> Any:
    """
    () -> array float{rblock,2}.
    Get a numpy array of the rblock applied force.
    """
    pass

def force_contact(*args, **kwargs) -> Any:
    """
    () -> array float{rblock,2}.
    Get a numpy array of the rblock contact force.
    """
    pass

def force_unbal(*args, **kwargs) -> Any:
    """
    () -> array float{rblock,2}.
    Get a numpy array of the rblock unbalanced force.
    """
    pass

def ids(*args, **kwargs) -> Any:
    """
    () -> array int{rblock}.
    Get the rblock ids as an array.
    """
    pass

def in_group(*args, **kwargs) -> Any:
    """
    (group_name: str, slot=1) -> array bool{rblock}.
    Return rblock group membership as a Boolean array.
    """
    pass

def mass(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the inertial rblock mass.
    """
    pass

def mass_real(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the real (gravitational) rblock mass.
    """
    pass

def moi_prin_real(*args, **kwargs) -> Any:
    """
    () -> array float{rblock,dim}.
    Get a numpy array of the rblock principal moments of inertia.
    """
    pass

def moment_app(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the rblock applied moment.
    """
    pass

def moment_contact(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the rblock contact moment.
    """
    pass

def moment_unbal(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the rblock unbalanced moment.
    """
    pass

def pos(*args, **kwargs) -> Any:
    """
    () -> array float{rblock,2}.
    Get a numpy array of the rblock centroid location.
    """
    pass

def rotation(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the rblock orientation.
    """
    pass

def set_damp(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Set the rblock local damping from an array.
    """
    pass

def set_density(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Set the rblock density from an array.
    """
    pass

def set_disp(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Set the rblock displacement from an array.
    """
    pass

def set_extra(*args, **kwargs) -> Any:
    """
    (slot: int, data: array float{rblock} or float{rblock}) -> None.
    Set the rblock extra data in the given slot with an array.
    Extra variables set by array must be of type float or vec.
    """
    pass

def set_force_app(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Set the rblock applied force from an array.
    """
    pass

def set_force_contact(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Set the rblock contact force from an array.
    """
    pass

def set_group(*args, **kwargs) -> Any:
    """
    (membership: array bool{rblock}, group_name: str, slot=1) -> None.
    Set rblock group from an array.
    Where membership True set the corresponding rblock to be a member of group group_name in the given slot.
    """
    pass

def set_moi_prin_real(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,dim}) -> None.
    Set the rblock principal moments of inertia from an array.
    When modified no other rblock attributes are changed (e.g., pebble sizes, rblock volume, etc.).
    The specification of the moment of inertia in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a rblock unless the user changes the fix state.
    """
    pass

def set_moment_app(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Set the rblock applied moment from an array.
    """
    pass

def set_moment_contact(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Set the rblock contact moment from an array.
    """
    pass

def set_pos(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Set the rblock centroid location from an array.
    """
    pass

def set_rotation(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Set the rblock orientation from an array.
    """
    pass

def set_spin(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Set the rblock angular velocity from an array.
    """
    pass

def set_vel(*args, **kwargs) -> Any:
    """
    (data: array float{rblock,2}) -> None.
    Set the rblock velocity from an array.
    """
    pass

def set_vol(*args, **kwargs) -> Any:
    """
    (data: array float{rblock}) -> None.
    Set the rblock volume from an array.
    In 2D this is the volume per unit thickness.
    When modified no other rblock attributes are changed (e.g., moment of inertia, pebble sizes, etc.).
    The specification of the volume in this way results in the principal moments of inertia being in a fixed state so that they will not be automatically updated when scaling a rblock unless the user changes the fix state.
    """
    pass

def spin(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the rblock angular velocity.
    """
    pass

def vel(*args, **kwargs) -> Any:
    """
    () -> array float{rblock,2}.
    Get a numpy array of the rblock velocity.
    """
    pass

def vol(*args, **kwargs) -> Any:
    """
    () -> array float{rblock}.
    Get a numpy array of the rblock volume.
    """
    pass

